#
# Copyright (C) 2018  FreeIPA Contributors see COPYING for license
#

"""
Module tests interactive command behavior
"""

from tempfile import NamedTemporaryFile
import textwrap

from ipatests.test_integration.base import IntegrationTest
from ipatests.pytest_plugins.integration import tasks


class TestInteractive(IntegrationTest):
    @classmethod
    def install(cls, mh):
        tasks.install_master(cls.master, setup_dns=False)

    def test_failed_user_add_password(self):
        """
        Non-matching interactive passwords

        This should cause the user to not be added.
        """
        pwd = "foo\nbar\n"
        user_login = 'tdoe'

        tasks.kinit_admin(self.master)
        self.master.run_command(['ipa', 'user-add', user_login,
                                 '--first', 'Tom',
                                 '--last', 'Doe',
                                 '--password'],
                                 stdin_text=pwd,
                                 raiseonerr=False)

        result = self.master.run_command(['ipa', 'user-show', user_login])
        assert '%s: user not found' % user_login in result.stderr_text

    def test_correct_user_add_password(self):
        """
        Base case: adding user with matching passwords
        """
        pwd = "secretpassword\nsecretpassword\n"
        user_login = 'jdoe'

        tasks.kinit_admin(self.master)
        self.master.run_command(['ipa', 'user-add', user_login,
                                 '--first', 'John',
                                 '--last', 'Doe',
                                 '--password'],
                                 stdin_text=pwd)

        result = self.master.run_command(['ipa', 'user-show', user_login])
        assert 'Added user "%s"' % user_login in result.stderr_text

    def test_change_sysaccount_password(self):
        """
        Use ldappasswd to change a system account user password
        """
        sysuser = 'system'
        original_passwd = 'Secret123'
        new_passwd = 'userPasswd123'

        base_dn = "dc=%s" % (",dc=".join(self.master.domain.name.split(".")))
        tf = NamedTemporaryFile()
        ldif_file = tf.name
        entry_ldif = textwrap.dedent("""
            dn: uid=system,cn=sysaccounts,cn=etc,{base_dn}
            changetype: add
            objectclass: account
            objectclass: simplesecurityobject
            uid: system
            userPassword: {original_passwd}
            passwordExpirationTime: 20380119031407Z
            nsIdleTimeout: 0""").format(
            base_dn=base_dn,
            original_passwd=original_passwd)
        self.master.put_file_contents(ldif_file, entry_ldif)
        arg = ['ldapmodify',
               '-h', self.master.hostname,
               '-p', '389', '-D',
               str(self.master.config.dirman_dn),   # pylint: disable=no-member
               '-w', self.master.config.dirman_password,
               '-f', ldif_file]
        self.master.run_command(arg)

        # change password for the user
        tasks.ldappasswd_sysaccount_change(sysuser, original_passwd,
                                           new_passwd, self.master)
